@@ -1,7 +1,7 @@
 /*
  * custom - interface for custom software and hardware interfaces
  *
- * Copyright (C) 1999-2007  Landon Curt Noll
+ * Copyright (C) 1999-2007,2021,2023  Landon Curt Noll
  *
  * Calc is open software; you can redistribute it and/or modify it under
  * the terms of the version 2.1 of the GNU Lesser General Public License
@@ -26,7 +26,7 @@
 
 /*
  * Be careful what you put in this file, upper .c files include
- * this file even when CUSTOM is not defined (ALLOW_CUSTOM is empty).
+ * this file even when CUSTOM is not defined (ALLOW_CUSTOM="-UCUSTOM").
  *
  * Don't include anything, let the including .c file bring in:
  *
@@ -35,7 +35,7 @@
  *
  * before they include this file.
  *
- * Keep this file down to a minimum.   Don't put custom builtin funcion
+ * Keep this file down to a minimum.   Don't put custom builtin function
  * stuff in this file!
  */
 
@@ -45,7 +45,7 @@
 
 
 /*
- * arg count definitons
+ * arg count definitions
  */
 #define MAX_CUSTOM_ARGS 100	/* maximum number of custom arguments */
 
@@ -73,11 +73,11 @@ struct custom {
  *
  * These are the required interfaces.  The dummy.c stubs these interfaces too.
  */
+E_FUNC bool custom_compiled(void);	/* return true if libcustcalc compiled CUSTOM defined, false otherwise */
 E_FUNC VALUE custom(char*, int, VALUE**);	/* master custom interface */
-EXTERN BOOL allow_custom;		/* TRUE => custom builtins allowed */
+EXTERN bool allow_custom;		/* true => custom builtins allowed */
 E_FUNC void showcustom(void);		/* print custom functions */
 E_FUNC void customhelp(char *);		/* direct custom help */
-EXTERN CONST struct custom cust[];	/* custom interface table */
 E_FUNC void init_custreg(void);		/* initialize custom registers */
 
 #endif /* !CUSTOM_H */
