@@ -1,7 +1,7 @@
 /*
  * zmath - declarations for extended precision integer arithmetic
  *
- * Copyright (C) 1999-2007,2014,2021  David I. Bell
+ * Copyright (C) 1999-2007,2014,2021,2023  David I. Bell and Landon Curt Noll
  *
  * Calc is open software; you can redistribute it and/or modify it under
  * the terms of the version 2.1 of the GNU Lesser General Public License
@@ -35,19 +35,25 @@
 
 
 #if defined(CALC_SRC)	/* if we are building from the calc source tree */
+# include "calc/version.h"
+# include "calc/bool.h"
 # include "calc/decl.h"
 # include "calc/alloc.h"
 # include "calc/endian_calc.h"
 # include "calc/longbits.h"
 # include "calc/byteswap.h"
 # include "calc/have_stdlib.h"
+# include "calc/charbit.h"
 #else
+# include <calc/version.h>
+# include <calc/bool.h>
 # include <calc/decl.h>
 # include <calc/alloc.h>
 # include <calc/endian_calc.h>
 # include <calc/longbits.h>
 # include <calc/byteswap.h>
 # include <calc/have_stdlib.h>
+# include <calc/charbit.h>
 #endif
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
@@ -55,16 +61,7 @@
 
 
 #ifndef ALLOCTEST
-# define freeh(p) { if (((void *)p != (void *)_zeroval_) &&		\
-			 ((void *)p != (void *)_oneval_)) free((void *)p); }
-#endif
-
-
-#if !defined(TRUE)
-#define TRUE	((BOOL) 1)			/* booleans */
-#endif
-#if !defined(FALSE)
-#define FALSE	((BOOL) 0)
+# define freeh(p) { if (!is_const(p)) { free((void *)(p)); } }
 #endif
 
 
@@ -82,11 +79,11 @@ typedef USB64 FULL;			/* double unit of number storage */
 typedef SB64 SFULL;			/* signed FULL */
 
 #define SWAP_HALF_IN_B64(dest, src)	SWAP_B32_IN_B64(dest, src)
-#define SWAP_HALF_IN_B32(dest, src)	(*(dest) = *(src))
+#define SWAP_HALF_IN_B32(dest, src)	(*((HALF *)(dest)) = *((HALF *)(src)))
 #define SWAP_HALF_IN_FULL(dest, src)	SWAP_B32_IN_B64(dest, src)
 #define SWAP_HALF_IN_HASH(dest, src)	SWAP_B16_IN_HASH(dest, src)
 #define SWAP_HALF_IN_FLAG(dest, src)	SWAP_B16_IN_FLAG(dest, src)
-#define SWAP_HALF_IN_BOOL(dest, src)	SWAP_B16_IN_BOOL(dest, src)
+#define SWAP_HALF_IN_bool(dest, src)	SWAP_B16_IN_bool(dest, src)
 #define SWAP_HALF_IN_LEN(dest, src)	SWAP_B16_IN_LEN(dest, src)
 #define SWAP_B32_IN_FULL(dest, src)	SWAP_B32_IN_B64(dest, src)
 #define SWAP_B16_IN_FULL(dest, src)	SWAP_B16_IN_B64(dest, src)
@@ -107,11 +104,11 @@ typedef SB32 SFULL;			/* signed FULL */
 #define SWAP_HALF_IN_FULL(dest, src)	SWAP_B16_IN_B32(dest, src)
 #define SWAP_HALF_IN_HASH(dest, src)	SWAP_B16_IN_HASH(dest, src)
 #define SWAP_HALF_IN_FLAG(dest, src)	SWAP_B16_IN_FLAG(dest, src)
-#define SWAP_HALF_IN_BOOL(dest, src)	SWAP_B16_IN_BOOL(dest, src)
+#define SWAP_HALF_IN_bool(dest, src)	SWAP_B16_IN_bool(dest, src)
 #define SWAP_HALF_IN_LEN(dest, src)	SWAP_B16_IN_LEN(dest, src)
-#define SWAP_B32_IN_FULL(dest, src)	(*(dest) = *(src))
+#define SWAP_B32_IN_FULL(dest, src)	(*((FULL *)(dest)) = *((FULL *)(src)))
 #define SWAP_B16_IN_FULL(dest, src)	SWAP_B16_IN_B32(dest, src)
-#define SWAP_B16_IN_HALF(dest, src)	(*(dest) = *(src))
+#define SWAP_B16_IN_HALF(dest, src)	(*((HALF *)(dest)) = *((HALF *)(src)))
 #define SWAP_B8_IN_FULL(dest, src)	SWAP_B8_IN_B32(dest, src)
 #define SWAP_B8_IN_HALF(dest, src)	SWAP_B8_IN_B16(dest, src)
 
@@ -124,6 +121,7 @@ typedef SB32 SFULL;			/* signed FULL */
 #define HALF_LEN (sizeof(HALF))			/* length of HALF in bites */
 #define FULL_LEN (sizeof(FULL))			/* length of FULL in bites */
 
+
 /*
  * ROUNDUP(value, mult) - round up value to the next multiple of mult
  *
@@ -150,7 +148,7 @@ typedef SB32 SFULL;			/* signed FULL */
 
 
 /*
- * other misc typedefs
+ * other miscellaneous typedefs
  */
 typedef USB32 QCKHASH;			/* 32 bit hash value */
 #if defined(HAVE_B64) && LONG_BITS == 32
@@ -163,22 +161,21 @@ typedef FULL PRINT;			/* cast for zio printing functions */
 #define SWAP_B8_IN_PRINT(dest, src)	SWAP_B8_IN_FULL(dest, src)
 #endif
 typedef SB32 FLAG;			/* small value (e.g. comparison) */
-typedef SB32 BOOL;			/* TRUE or FALSE value */
-typedef SB32 LEN;			/* unit of length storage */
 
-#define SWAP_B32_IN_HASH(dest, src)	(*(dest) = *(src))
-#define SWAP_B16_IN_HASH(dest, src)	SWAP_B16_IN_B32(dest, src)
-#define SWAP_B8_IN_HASH(dest, src)	SWAP_B8_IN_B32(dest, src)
+/*
+ * length of internal integer values in units of HALF
+ */
+#if MAJOR_VER < 3
+typedef SB32 LEN;			/* calc v2 compatible unit of length storage */
+#else /* MAJOR_VER < 3 */
+typedef uintptr_t LEN;			/* unit of length storage */
+#endif /* MAJOR_VER < 3 */
 
-#define SWAP_B32_IN_FLAG(dest, src)	(*(dest) = *(src))
-#define SWAP_B16_IN_FLAG(dest, src)	SWAP_B16_IN_B32(dest, src)
-#define SWAP_B8_IN_FLAG(dest, src)	SWAP_B8_IN_B32(dest, src)
+#define SWAP_B32_IN_bool(dest, src)	(*((bool *)(dest)) = *((bool *)(src)))
+#define SWAP_B16_IN_bool(dest, src)	SWAP_B16_IN_B32(dest, src)
+#define SWAP_B8_IN_bool(dest, src)	SWAP_B8_IN_B32(dest, src)
 
-#define SWAP_B32_IN_BOOL(dest, src)	(*(dest) = *(src))
-#define SWAP_B16_IN_BOOL(dest, src)	SWAP_B16_IN_B32(dest, src)
-#define SWAP_B8_IN_BOOL(dest, src)	SWAP_B8_IN_B32(dest, src)
-
-#define SWAP_B32_IN_LEN(dest, src)	(*(dest) = *(src))
+#define SWAP_B32_IN_LEN(dest, src)	(*((LEN *)(dest)) = *((LEN *)(src)))
 #define SWAP_B16_IN_LEN(dest, src)	SWAP_B16_IN_B32(dest, src)
 #define SWAP_B8_IN_LEN(dest, src)	SWAP_B8_IN_B32(dest, src)
 
@@ -234,9 +231,25 @@ typedef SB32 LEN;			/* unit of length storage */
 
 
 /*
- * LEN storage size must be <= FULL storage size
+ * MAXDATA - largest data object in bytes we will use
+ *
+ * We start with MAXDATA as 1/16 of the maximum address space.
+ * We limit to 1/16 because for a maximum complex value we
+ * will need 4 huge integers, plus other data, code and stack space.
  */
-#define MAXLEN	((LEN) 0x7fffffff >> 3) /* longest value allowed */
+#if MAJOR_VER < 3
+#define MAXDATA (0x80000000>>3)		/* calc v2 compatible supported address space */
+#else /* MAJOR_VER < 3 */
+#define MAXDATA ((LEN) 1<<(UINTPTR_WIDTH-4))	/* 1/16 of address space */
+#endif /* MAJOR_VER < 3 */
+
+/*
+ * MAXLEN - maximum length of internal integer values in units of HALF
+ *
+ * We limit MAXLEN based on 1 less than the number of HALFs that
+ * will fit into MAXDATA bytes.
+ */
+#define MAXLEN ((LEN) ((MAXDATA / HALF_LEN) - 1))	/* longest value allowed */
 
 
 #define MAXREDC 256			/* number of entries in REDC cache */
@@ -277,10 +290,19 @@ typedef union {
 #endif
 
 
+
+/*
+ * ZVALUE - multi-prevision integer
+ */
+#if MAJOR_VER < 3
+typedef SB32 SIGN;			/* calc v2 compatible sign type */
+#else /* MAJOR_VER < 3 */
+typedef bool SIGN;			/* sign as a C boolean */
+#endif /* MAJOR_VER < 3 */
 typedef struct {
 	HALF	*v;		/* pointer to array of values */
 	LEN	len;		/* number of values in array */
-	BOOL	sign;		/* sign, nonzero is negative */
+	SIGN	sign;		/* sign, nonzero is negative */
 } ZVALUE;
 
 
@@ -289,6 +311,7 @@ typedef struct {
  * Function prototypes for integer math routines.
  */
 E_FUNC HALF * alloc(LEN len);
+E_FUNC int is_const(HALF* h);
 #ifdef	ALLOCTEST
 E_FUNC void freeh(HALF *);
 #endif
@@ -325,7 +348,7 @@ E_FUNC long zdiv(ZVALUE z1, ZVALUE z2, ZVALUE *res, ZVALUE *rem, long R);
 E_FUNC long zquo(ZVALUE z1, ZVALUE z2, ZVALUE *res, long R);
 E_FUNC long zmod(ZVALUE z1, ZVALUE z2, ZVALUE *rem, long R);
 E_FUNC void zequo(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-E_FUNC BOOL zdivides(ZVALUE z1, ZVALUE z2);
+E_FUNC bool zdivides(ZVALUE z1, ZVALUE z2);
 E_FUNC void zor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
 E_FUNC void zand(ZVALUE z1, ZVALUE z2, ZVALUE *res);
 E_FUNC void zxor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
@@ -336,13 +359,13 @@ E_FUNC void zsquare(ZVALUE z, ZVALUE *res);
 E_FUNC long zlowbit(ZVALUE z);
 E_FUNC LEN zhighbit(ZVALUE z);
 E_FUNC void zbitvalue(long n, ZVALUE *res);
-E_FUNC BOOL zisset(ZVALUE z, long n);
-E_FUNC BOOL zisonebit(ZVALUE z);
-E_FUNC BOOL zisallbits(ZVALUE z);
+E_FUNC bool zisset(ZVALUE z, long n);
+E_FUNC bool zisonebit(ZVALUE z);
+E_FUNC bool zisallbits(ZVALUE z);
 E_FUNC FLAG ztest(ZVALUE z);
 E_FUNC FLAG zrel(ZVALUE z1, ZVALUE z2);
 E_FUNC FLAG zabsrel(ZVALUE z1, ZVALUE z2);
-E_FUNC BOOL zcmp(ZVALUE z1, ZVALUE z2);
+E_FUNC bool zcmp(ZVALUE z1, ZVALUE z2);
 
 
 /*
@@ -361,10 +384,10 @@ E_FUNC void zfib(ZVALUE z, ZVALUE *res);
 E_FUNC void zpowi(ZVALUE z1, ZVALUE z2, ZVALUE *res);
 E_FUNC void ztenpow(long power, ZVALUE *res);
 E_FUNC void zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res);
-E_FUNC BOOL zmodinv(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-E_FUNC BOOL zrelprime(ZVALUE z1, ZVALUE z2);
+E_FUNC bool zmodinv(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+E_FUNC bool zrelprime(ZVALUE z1, ZVALUE z2);
 E_FUNC long zlog(ZVALUE z1, ZVALUE z2);
-E_FUNC long zlog10(ZVALUE z, BOOL *was_10_power);
+E_FUNC long zlog10(ZVALUE z, bool *was_10_power);
 E_FUNC long zdivcount(ZVALUE z1, ZVALUE z2);
 E_FUNC long zfacrem(ZVALUE z1, ZVALUE z2, ZVALUE *rem);
 E_FUNC long zgcdrem(ZVALUE z1, ZVALUE z2, ZVALUE *res);
@@ -372,8 +395,9 @@ E_FUNC long zdigits(ZVALUE z1);
 E_FUNC long zdigit(ZVALUE z1, long n);
 E_FUNC FLAG zsqrt(ZVALUE z1, ZVALUE *dest, long R);
 E_FUNC void zroot(ZVALUE z1, ZVALUE z2, ZVALUE *dest);
-E_FUNC BOOL zissquare(ZVALUE z);
+E_FUNC bool zissquare(ZVALUE z);
 E_FUNC void zhnrmod(ZVALUE v, ZVALUE h, ZVALUE zn, ZVALUE zr, ZVALUE *res);
+E_FUNC bool zispowerof2(ZVALUE z, FULL *log2);
 
 
 /*
@@ -384,11 +408,11 @@ E_FUNC FULL znprime(ZVALUE z);
 E_FUNC FULL next_prime(FULL v);
 E_FUNC FULL zpprime(ZVALUE z);
 E_FUNC void zpfact(ZVALUE z, ZVALUE *dest);
-E_FUNC BOOL zprimetest(ZVALUE z, long count, ZVALUE skip);
-E_FUNC BOOL zredcprimetest(ZVALUE z, long count, ZVALUE skip);
-E_FUNC BOOL znextcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res,
+E_FUNC bool zprimetest(ZVALUE z, long count, ZVALUE skip);
+E_FUNC bool zredcprimetest(ZVALUE z, long count, ZVALUE skip);
+E_FUNC bool znextcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res,
 		      ZVALUE mod, ZVALUE *cand);
-E_FUNC BOOL zprevcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res,
+E_FUNC bool zprevcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res,
 		      ZVALUE mod, ZVALUE *cand);
 E_FUNC FULL zlowfactor(ZVALUE z, long count);
 E_FUNC FLAG zfactor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
@@ -397,11 +421,11 @@ E_FUNC void zlcmfact(ZVALUE z, ZVALUE *dest);
 
 
 /*
- * Misc misc functions. :-)
+ * miscellaneous functions. :-)
  */
 E_FUNC void zsquaremod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
 E_FUNC void zminmod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-E_FUNC BOOL zcmpmod(ZVALUE z1, ZVALUE z2, ZVALUE z3);
+E_FUNC bool zcmpmod(ZVALUE z1, ZVALUE z2, ZVALUE z3);
 E_FUNC void zio_init(void);
 
 
@@ -457,18 +481,18 @@ E_FUNC void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 #define zistiny(z)	((z).len == 1)
 
 /*
- * zgtmaxfull(z)	TRUE if abs(z) > MAXFULL
+ * zgtmaxfull(z)	true if abs(z) > MAXFULL
  */
 #define zgtmaxfull(z)	(((z).len > 2) || (((z).len == 2) && \
 			 (((SHALF)(z).v[1]) < 0)))
 
 /*
- * zgtmaxufull(z)	TRUE if abs(z) will not fit into a FULL (> MAXUFULL)
+ * zgtmaxufull(z)	true if abs(z) will not fit into a FULL (> MAXUFULL)
  */
 #define zgtmaxufull(z)	((z).len > 2)
 
 /*
- * zgtmaxulong(z)	TRUE if abs(z) > MAXULONG
+ * zgtmaxulong(z)	true if abs(z) > MAXULONG
  */
 #if BASEB >= LONG_BITS
 #define zgtmaxulong(z)	((z).len > 1)
@@ -477,7 +501,7 @@ E_FUNC void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 #endif
 
 /*
- * zgtmaxlong(z)	TRUE if abs(z) > MAXLONG
+ * zgtmaxlong(z)	true if abs(z) > MAXLONG
  */
 #if BASEB >= LONG_BITS
 #define zgtmaxlong(z)	(((z).len > 1) || (((z).len == 1) && \
@@ -493,18 +517,18 @@ E_FUNC void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
  * against a different power of 2 on a 64 bit machine.	The macros below
  * provide a tests against powers of 2 that are independent of the work size.
  *
- *	zge16b(z)	TRUE if abs(z) >= 2^16
- *	zge24b(z)	TRUE if abs(z) >= 2^24
- *	zge31b(z)	TRUE if abs(z) >= 2^31
- *	zge32b(z)	TRUE if abs(z) >= 2^32
- *	zge64b(z)	TRUE if abs(z) >= 2^64
- *	zge128b(z)	TRUE if abs(z) >= 2^128
- *	zge256b(z)	TRUE if abs(z) >= 2^256
- *	zge512b(z)	TRUE if abs(z) >= 2^512
- *	zge1024b(z)	TRUE if abs(z) >= 2^1024
- *	zge2048b(z)	TRUE if abs(z) >= 2^2048
- *	zge4096b(z)	TRUE if abs(z) >= 2^4096
- *	zge8192b(z)	TRUE if abs(z) >= 2^8192
+ *	zge16b(z)	true if abs(z) >= 2^16
+ *	zge24b(z)	true if abs(z) >= 2^24
+ *	zge31b(z)	true if abs(z) >= 2^31
+ *	zge32b(z)	true if abs(z) >= 2^32
+ *	zge64b(z)	true if abs(z) >= 2^64
+ *	zge128b(z)	true if abs(z) >= 2^128
+ *	zge256b(z)	true if abs(z) >= 2^256
+ *	zge512b(z)	true if abs(z) >= 2^512
+ *	zge1024b(z)	true if abs(z) >= 2^1024
+ *	zge2048b(z)	true if abs(z) >= 2^2048
+ *	zge4096b(z)	true if abs(z) >= 2^4096
+ *	zge8192b(z)	true if abs(z) >= 2^8192
  */
 #if BASEB == 32
 
@@ -579,7 +603,12 @@ E_FUNC void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 #define zcopyval(z1,z2) memcpy((z2).v, (z1).v, (z1).len * sizeof(HALF))
 #define zquicktrim(z)	{if (((z).len > 1) && ((z).v[(z).len-1] == 0)) \
 				(z).len--;}
-#define zfree(z)	freeh((z).v)
+#define zfree(z)	{if ((z).len != 0 && (z).v != NULL) { \
+			    freeh((z).v); \
+			    (z).v = NULL; \
+			    (z).len = 0; \
+			    (z).sign = 0; } \
+			}
 
 
 /*
@@ -598,12 +627,7 @@ E_FUNC void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 #define MODE_MAX	9
 #define MODE2_OFF	(MODE_MAX+1)
 
-/* XXX - perhaps we need the MODE_REAL_AUTO vs MODE_REAL as a config mode? */
-#if 0 	/* XXX - can we safely set MODE_INITIAL to MODE_REAL_AUTO ?? */
-#define MODE_INITIAL	MODE_REAL_AUTO
-#else
 #define MODE_INITIAL	MODE_REAL
-#endif
 #define MODE2_INITIAL	MODE2_OFF
 
 
@@ -620,44 +644,54 @@ E_FUNC char *math_getdivertedio(void);
 E_FUNC int math_setmode(int mode);
 E_FUNC int math_setmode2(int mode);
 E_FUNC LEN math_setdigits(LEN digits);
-E_FUNC void math_fmt(char *, ...) PRINTF_FORMAT(1, 2);
-
-
-/*
- * The error routine.
- */
-E_FUNC void math_error(char *, ...) PRINTF_FORMAT(1, 2);
+E_FUNC void math_fmt(char *, ...) __attribute__((format(printf, 1, 2)));
 
 
 /*
  * external swap functions
  */
 E_FUNC HALF *swap_b8_in_HALFs(HALF *dest, HALF *src, LEN len);
-E_FUNC ZVALUE *swap_b8_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
+E_FUNC ZVALUE *swap_b8_in_ZVALUE(ZVALUE *dest, ZVALUE *src, bool all);
 E_FUNC HALF *swap_b16_in_HALFs(HALF *dest, HALF *src, LEN len);
-E_FUNC ZVALUE *swap_b16_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
-E_FUNC ZVALUE *swap_HALF_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
+E_FUNC HALF *swap_HALFs(HALF *dest, HALF *src, LEN len);
+E_FUNC ZVALUE *swap_b16_in_ZVALUE(ZVALUE *dest, ZVALUE *src, bool all);
+E_FUNC ZVALUE *swap_HALF_in_ZVALUE(ZVALUE *dest, ZVALUE *src, bool all);
 
 
 /*
  * constants used often by the arithmetic routines
  */
-EXTERN HALF _zeroval_[], _oneval_[], _twoval_[], _threeval_[], _fourval_[];
-EXTERN HALF _fiveval_[], _sixval_[], _sevenval_[], _eightval_[], _nineval_[];
-EXTERN HALF _tenval_[], _elevenval_[], _twelveval_[], _thirteenval_[];
-EXTERN HALF _fourteenval_[], _fifteenval_[];
+EXTERN HALF _zeroval_[];
+EXTERN ZVALUE _zero_;
+
+EXTERN HALF _oneval_[];
+EXTERN ZVALUE _one_;
+EXTERN ZVALUE _neg_one_;
+
+EXTERN HALF _twoval_[];
+EXTERN ZVALUE _two_;
+
+EXTERN HALF _tenval_[];
+EXTERN ZVALUE _ten_;
+
 EXTERN HALF _sqbaseval_[];
-EXTERN HALF _fourthbaseval_[];
-EXTERN HALF _threesixtyval_[], _fourhundredval_[], _twentyfourval_[];
+EXTERN ZVALUE _sqbase_;
 
-EXTERN ZVALUE zconst[];		/* ZVALUE integers from 0 thru 15 */
+EXTERN HALF _pow4baseval_[];
+EXTERN ZVALUE _pow4base_;
 
-EXTERN ZVALUE _zero_, _one_, _two_, _ten_, _neg_one_;
-EXTERN ZVALUE _sqbase_, _pow4base_, _pow8base_;
+EXTERN HALF _pow8baseval_[];
+EXTERN ZVALUE _pow8base_;
 
-EXTERN ZVALUE _b32_, _b64_;
+/* _b32_ is _sqbaseval_ or _pow4baseval_ depending on BASEB */
+EXTERN ZVALUE _b32_;
 
-EXTERN BOOL _math_abort_;	/* nonzero to abort calculations */
+/* _b64_ is _pow4baseval_ or _pow8baseval_ depending on BASEB */
+EXTERN ZVALUE _b64_;
+
+EXTERN HALF *half_tbl[]; /* preset HALF constants, NULL terminated list */
+
+EXTERN bool _math_abort_;	/* nonzero to abort calculations */
 EXTERN ZVALUE _tenpowers_[];	/* table of 10^2^n */
 
 /*
